const wxService = require('./wxService')
const moment = require('../lib/moment.min')
const Promise = require('../lib/bluebird')
const _ = require('../lib/underscore.min')
const formatTime = date => {
  const year = date.getFullYear()
  const month = date.getMonth() + 1
  const day = date.getDate()
  const hour = date.getHours()
  const minute = date.getMinutes()
  const second = date.getSeconds()

  return [year, month, day].map(formatNumber).join('/') + ' ' + [hour, minute, second].map(formatNumber).join(':')
}

/* 获取当前页url */
function getCurrentPageUrl() {
  var pages = getCurrentPages() // 获取加载的页面
  var currentPage = pages[pages.length - 1] // 获取当前页面的对象
  var url = currentPage.route // 当前页面url
  return url
}

/* 获取当前页带参数的url */
function getCurrentPageUrlWithArgs() {
  var pages = getCurrentPages() // 获取加载的页面
  var currentPage = pages[pages.length - 1] // 获取当前页面的对象
  var url = currentPage.route // 当前页面url
  var options = currentPage.options // 如果要获取url中所带的参数可以查看options

  // 拼接url的参数
  var urlWithArgs = url + '?'
  for (var key in options) {
    var value = options[key]
    urlWithArgs += key + '=' + value + '&'
  }
  urlWithArgs = urlWithArgs.substring(0, urlWithArgs.length - 1)

  return urlWithArgs
}

function getCurrentRouter() {
  var pages = getCurrentPages() // 获取加载的页面
  var currentPage = pages[pages.length - 1] // 获取当前页面的对象
  return currentPage
}

function getNowTime() {
  return wxService.getManageDate().getNow()
}

function getAppUserInfo() {
  const app = getApp()
  if (app.globalData.userInfo) {
    return Promise.resolve({ userInfo: app.globalData.userInfo })
  } else {
    return Promise.reject('userinfo is null')
  }
}

/**
 * 处理分页数据安全加载,渲染
 * 小程序1M的限制
 */
class HandlerDataOnPage {
  init(target, originArrName) {
    if (target && target.data && originArrName) {
      this.target = target.data
      this.OriginArrayName = originArrName
    }
  }

  handlerArray(dataArray = []) {
    const target = this.target
    const arrayName = this.OriginArrayName
    dataArray = dataArray || []
    this.validate()
    if (toString.call(dataArray) !== '[object Array]') {
      throw new Error('dataArray invaild')
    }

    let len = target[arrayName].length
    target[arrayName] = target[arrayName].concat(dataArray)

    return this.trunObject(dataArray, len)
  }

  editArray(dataArray = [], startIndex = -1) {
    if (startIndex < 0) {
      throw new Error('editArray startIndex must > -1')
    }
    return this.trunObject(dataArray, startIndex)
  }

  trunObject(dataArray = [], len) {
    const arrayName = this.OriginArrayName
    function RenderArray2Object() { };
    RenderArray2Object.prototype.keysNum = function () {
      return Object.keys(this).length
    }
    let renderArray2Object = new RenderArray2Object()
    for (let i = 0, s = dataArray.length; i < s; i++) {
      renderArray2Object[`${arrayName}[${len + i}]`] = dataArray[i]
    }
    return renderArray2Object
  }

  validate() {
    const target = this.target; const arrayName = this.OriginArrayName
    target[arrayName] = target[arrayName] || []
    if (!target) {
      throw new Error('target invaild')
    }
    if (!(target[arrayName] && toString.call(target[arrayName]) === '[object Array]')) {
      target[arrayName] = []
    }
  }

  clearOriginArr() {
    const target = this.target; const arrayName = this.OriginArrayName
    this.validate()
    target[arrayName] = []
  }
}

class Logger {
  log() {
    // if (envInfo.env === 'prod') {
    //   return
    // }
    let args = Array.prototype.slice.apply(arguments)

    let params = Array.prototype.slice.apply(arguments)
    const desc = args.shift()
    let tip = `======= ${desc} =======`
    args.forEach(() => {
      tip += ',%o'
    })
    params[0] = tip
    console.log.apply(console, params)
  }
}

// 通过点击按钮 获取用户信息并登陆注册到crm系统中
function getUserInfoByBtn(userInfo) {
  const logger = new Logger()
  if (Object.keys(userInfo).length === 1) {
    return wxService.Promise.reject(userInfo.errMsg)
  }
  return wxService.login().then((res) => {
    logger.log(' wx getUserInfoByBtn success res --->', res)
    return Promise.all([Promise.resolve(userInfo), Promise.resolve(res)])
  }).then((res) => {
    logger.log(' wx _userInfo success res --->', res)
    wx.setStorageSync('_userInfo', res[0].userInfo)
    wx.showLoading({
      title: '正在加载...',
      icon: 'none',
      mask: true
    })
    return wxService.post(`/member/minaLogin`, {
      code: res[1].code,
      wechatInfo: res[0]
    })
  }, (err) => {
    return Promise.reject(err)
  }).then(response => {
    console.log(JSON.stringify(response))
    logger.log('getCrmUserInfo response', response)
    if (response.data.data.rspCode === -1) {
      return Promise.reject(response.data.data)
    }
    return Promise.resolve(response.data.data)
  }).finally(() => {
    wx.hideLoading()
  })
}


const formatNumber = n => {
  n = n.toString()
  return n[1] ? n : '0' + n
}

module.exports = {
  formatTime,
  getCurrentPageUrl,
  getCurrentRouter,
  getCurrentPageUrlWithArgs,
  getNowTime,
  getUserInfoByBtn,
  moment,
  _,
  // _: underscore,
  HandlerDataOnPage,
  formatTime: formatTime,
  getAppUserInfo,
  formatNumber: formatNumber,
  logger: new Logger()
}
